import * as fs from 'fs';
import log4js from 'log4js';
import EventEmitter from 'events';
const logger = log4js.getLogger();
export default class SettingStore extends EventEmitter {
    constructor() {
        super();
        this.state = {
            adapter: 'mysql',
            host: null,
            port: null,
            user: null,
            database: null,
            password: null,
            ssh: null
        };
    }
    static getInstance() {
        if (SettingStore.instance) {
            return SettingStore.instance;
        }
        SettingStore.instance = new SettingStore();
        return SettingStore.instance;
    }
    getSetting() {
        return Object.assign({}, this.state);
    }
    setSettingFromFile(path) {
        return new Promise((resolve, reject) => {
            fs.stat(path, (err, _stat) => {
                if (err) {
                    if (err.code && err.code === 'ENOENT') {
                        logger.debug("there isn't config file.");
                        resolve();
                        return;
                    }
                    logger.error(err.message);
                    reject(err);
                    return;
                }
                fs.readFile(path, { encoding: 'utf8' }, (err, data) => {
                    if (err) {
                        logger.error(err.message);
                        reject(err);
                        return;
                    }
                    this.setSetting(JSON.parse(data));
                    logger.debug(`read settings from ${path}`);
                    logger.debug(JSON.stringify(this.getSetting()));
                    resolve(this.getSetting());
                    return;
                });
            });
        });
    }
    hasSSHConfig() {
    }
    setSetting(setting) {
        this.state = Object.assign({}, this.state, setting);
        this.emit('change', this.state);
    }
}
//# sourceMappingURL=SettingStore.js.map