import log4js from 'log4js';
import { SSHConnection } from 'node-ssh-forward';
const logger = log4js.getLogger();
export default class AbstractClient {
    constructor(settings) {
        this.settings = settings;
    }
    async getSchema() {
        var _a, _b;
        let schema = [];
        this.connect();
        const sshConnection = this.settings.ssh ? new SSHConnection({ endHost: this.settings.ssh.remoteHost }) : null;
        if (sshConnection) {
            await sshConnection.forward({
                fromPort: ((_a = this.settings.ssh) === null || _a === void 0 ? void 0 : _a.port) || this.DefaultPort,
                toPort: this.settings.port || this.DefaultPort,
                toHost: ((_b = this.settings.ssh) === null || _b === void 0 ? void 0 : _b.dbHost) || '127.0.0.1'
            });
        }
        try {
            const tables = await this.getTables();
            schema = await Promise.all(tables.map((v) => this.getColumns(v).then(columns => ({
                database: this.settings.database,
                tableName: v,
                columns: columns.map(v => this.toColumnFromRawField(v))
            }))));
        }
        catch (e) {
            logger.error(e);
            throw e;
        }
        finally {
            this.disconnect();
            if (sshConnection) {
                sshConnection.shutdown();
            }
        }
        return schema;
    }
    toColumnFromRawField(field) {
        return {
            columnName: field.field,
            description: `${field.field}(Type: ${field.type}, Null: ${field.null}, Default: ${field.default})`
        };
    }
}
//# sourceMappingURL=AbstractClient.js.map