import * as mysql from 'mysql';
import AbstractClient from './AbstractClient';
export default class MysqlClient extends AbstractClient {
    constructor(settings) {
        super(settings);
        this.connection = null;
    }
    get DefaultPort() { return 3306; }
    get DefaultHost() { return '127.0.0.1'; }
    get DefaultUser() { return 'root'; }
    connect() {
        this.connection = mysql.createConnection({
            host: this.settings.host || this.DefaultHost,
            password: this.settings.password || '',
            user: this.settings.user || this.DefaultUser,
            port: this.settings.port || this.DefaultPort,
            database: this.settings.database || ''
        });
    }
    disconnect() {
        if (this.connection) {
            this.connection.end();
        }
        this.connection = null;
    }
    getTables() {
        const sql = `
      SELECT table_name 
      FROM information_schema.tables
      WHERE table_schema = '${this.settings.database}'
    `;
        return new Promise((resolve, reject) => {
            if (!this.connection) {
                reject(new Error("Don't have database connection."));
                return;
            }
            this.connection.query(sql, (err, results) => {
                if (err) {
                    reject(new Error(err.message));
                    return;
                }
                const tables = results.map((v) => v[`table_name`]);
                resolve(tables);
            });
        });
    }
    getColumns(tableName) {
        const sql = `SHOW FULL FIELDS FROM ${tableName}`;
        return new Promise((resolve, reject) => {
            if (!this.connection) {
                reject(new Error("Don't have database connection."));
                return;
            }
            this.connection.query(sql, (err, results) => {
                if (err) {
                    reject(new Error(err.message));
                    return;
                }
                const columns = results.map((v) => ({
                    field: v.Field,
                    type: v.Type,
                    null: v.Null,
                    default: v.Default,
                    comment: v.Comment
                }));
                resolve(columns);
            });
        });
    }
}
//# sourceMappingURL=MysqlClient.js.map